# encoding: utf-8
"""Manage registry of OAI-PMH providers.

usage: oai-reg [-h] [-d DATABASEPATH] {add,rm,list} ...

positional arguments:
  {add,rm,list}         Actions
    add                 Add a new OAI-PMH provider
    rm                  Remove a registered OAI-PMH provider
    list                List registered OAI-PMH provider

optional arguments:
  -h, --help            show this help message and exit
  -d DATABASEPATH, --database DATABASEPATH
                        Path to provider registry database. Currently supports
                        sqlite3 only.

Copyright © 2013, the University of Liverpool <http://www.liv.ac.uk>. All
rights reserved.

Distributed under the terms of the BSD 3-clause License
<http://opensource.org/licenses/BSD-3-Clause>.
"""

import logging
import os
import sqlite3
import sys

from argparse import ArgumentParser
from datetime import datetime

# Import oaipmh for validation purposes
from oaipmh.client import Client
from oaipmh.metadata import MetadataRegistry, oai_dc_reader
from oaipmh.error import XMLSyntaxError
from urllib2 import HTTPError


MAX_NAME_LENGTH = 15


def add_provider(cxn, args):
    """Add a new provider to the registry database.
    
    Process ``args`` to add a new provider to the registry database. Return 0
    for success, 1 for failure (error message should be logged).
    
    ``cxn`` => instance of ``sqlite3.Connection``
    ``args`` => instance of ``argparse.Namespace``
    """
    global logger, MAX_NAME_LENGTH
    addlogger = logger.getChild('add')
    # Validate name
    if len(args.name) > MAX_NAME_LENGTH:
        addlogger.critical('Short name for new provider must be no more than '
                           '{0} characters long'.format(MAX_NAME_LENGTH))
        return 1
    elif args.name.startswith(('http://', 'https://')) or args.name == 'all':
        addlogger.critical('Short name for new provider may not be "all" nor '
                           'may it begin "http://" or "https://"')
        return 1
    # Try to create row now to avoid unnecessary validation if duplicate
    try:
        cxn.execute("INSERT INTO providers(name, lastHarvest) values "
                         "(?, ?)",
                         (args.name, datetime.fromtimestamp(0))
        )
    except sqlite3.IntegrityError:
        addlogger.critical('Unable to add provider "{0}"; '
                           'provider with this name already exists'
                           ''.format(args.name)
                           )
        return 1
    else:
        addlogger.info('Adding provider "{0}"'.format(args.name))
    # Get any missing information
    # Base URL
    if args.url is None:
        args.url = raw_input('Base URL:'.ljust(20))
        if not args.url:
            addlogger.critical('Base URL for new provider not supplied')
            return 1
    # Set up an OAI-PMH client for validating providers
    md_registry = MetadataRegistry()
    md_registry.registerReader('oai_dc', oai_dc_reader)
    client = Client(args.url, md_registry)
    # Validate Base URL by fetching Identify
    try:
        client.identify()
    except (XMLSyntaxError, HTTPError):
        addlogger.critical('Base URL for new provider does not return a valid '
                           'response to an `Identify` request')
        return 1
    # Destination
    if args.dest is None:
        args.dest = raw_input('Destination directory: '.ljust(20))
        if args.dest:
            # Expand user dir
            args.dest = os.path.expanduser(args.dest)
        else:
            addlogger.info('Destination for data for new provider not supplied'
                           ' using default `pwd`: {0}'.format(os.getcwd())
                           )
            args.dest = os.getcwd()
    # metadataPrefix
    # Check that selected metadataPrefix is available from provider
    # Fetch list of available formats
    mdps = dict((mdpinfo[0], mdpinfo[1:])
                    for mdpinfo in
                    client.listMetadataFormats())
    while args.metadataPrefix not in mdps:
        print "Available metadataPrefix values:"
        # List available formats
        for mdp in mdps:
            print mdp, '-', mdps[mdp][1]
        args.metadataPrefix = raw_input('metadataPrefix [oai_dc]:'.ljust(20))
        if not args.metadataPrefix:
            addlogger.info('metadataPrefix for new provider not supplied. '
                           'using default: oai_dc')
            args.metadataPrefix = 'oai_dc'
    cxn.execute("UPDATE providers SET "
                     "url=?, "
                     "destination=?, "
                     "metadataPrefix=? "
                     "WHERE name=?",
                     (args.url,
                      args.dest,
                      args.metadataPrefix,
                      args.name
                      )
    )
    addlogger.info('URL for next harvest: {0}?verb=ListRecords'
                   '&metadataPrefix={1}'
                   '&from={2:%Y-%m-%dT%H:%M:%SZ%z}'
                   ''.format(args.url,
                             args.metadataPrefix,
                             datetime.fromtimestamp(0)
                             )
                   )
    # All done, commit database
    cxn.commit()
    return 0


def rm_provider(cxn, args):
    """Remove existing provider(s) from the registry database.
    
    Process ``args`` to remove provider(s) to the registry database. Return 0
    for success, 1 for failure (error message should be logged).
    
    ``cxn`` => instance of ``sqlite3.Connection``
    ``args`` => instance of ``argparse.Namespace``
    """
    global logger
    rmlogger = logger.getChild('remove')
    for name in args.name:
        with cxn:
            cur = cxn.execute(
                                   "DELETE FROM providers WHERE name=?",
                                   (name,)
                                   )
            if cur.rowcount <= 0:
                rmlogger.error(
                    'No provider named "{0}"; not deleted'.format(name)
                )
            else:
                rmlogger.info('Deleted provider "{0}"'.format(name))
    return 0


def list_providers(cxn, args):
    """List provider(s) currently in the registry database.
    
    Process ``args`` to remove provider(s) to the registry database. Return 0
    for success, 1 for failure (error message should be logged).
    
    ``cxn`` => instance of ``sqlite3.Connection``
    ``args`` => instance of ``argparse.Namespace``
    """
    global logger, MAX_NAME_LENGTH
    listlogger = logger.getChild('remove')
    if args.url:
        sql = 'SELECT name, url FROM providers'
        label = 'Base URL'
    elif args.dest:
        sql = 'SELECT name, destination FROM providers'
        label = "Destination"
    elif args.metadataPrefix:
        sql = 'SELECT name, metadataPrefix FROM providers'
        label = "metadataPrefix"
    elif args.lastHarvest:
        sql = 'SELECT name, lastHarvest FROM providers'
        label = "Last Completed Harvest Time"
    else:
        # Default is smart URL for next harvest request
        sql = ("SELECT name, url || "
               "'?verb=ListRecords&metadataPrefix=' || "
               "metadataPrefix || '&from=' || lastHarvest "
               "FROM providers")
        label = 'URL for next harvest'
    cursor = cxn.execute(sql)
    sys.stdout.write(''.join(['name'.ljust(MAX_NAME_LENGTH + 1),
                              label, '\n']))
    sys.stdout.write(' '.join(['=' * MAX_NAME_LENGTH, '=' * len(label), '\n']))
    for row in cursor:
        sys.stdout.write('{0:<{width}} {1}\n'.format(row[0],
                                                     row[1],
                                                     width=MAX_NAME_LENGTH))
        sys.stdout.flush()


def verify_database(path):
    """Verify that a suitable database exists, create it if not."""
    global logger
    var_logger = logger.getChild('verify')
    try:
        cxn = sqlite3.connect(
            path,
            detect_types=sqlite3.PARSE_DECLTYPES|sqlite3.PARSE_COLNAMES
        )
    except sqlite3.OperationalError:
        var_logger.critical('Database file "{0}" does not exist and cannot be '
                            'created'.format(path))
        return 1
    # Verify that table exists
    try:
        
        cxn.execute('SELECT name FROM providers')
    except sqlite3.OperationalError:
        # Create the table
        cxn.execute("CREATE TABLE providers("
                    "id integer primary key, "
                    "name varchar({0}) unique, "
                    "url varchar, "
                    "destination varchar, "
                    "metadataPrefix varchar, "
                    "lastHarvest timestamp)".format(MAX_NAME_LENGTH)
                    )
    return cxn


def main(argv=None):
    '''Process command line options, hand off to appropriate function.'''
    global argparser, connection, logger
    if argv is None:
        args = argparser.parse_args()
    else:
        args = argparser.parse_args(argv)
    cxn = verify_database(args.databasePath)
    if not isinstance(cxn, sqlite3.Connection):
        return cxn
    cxn = cxn
    return args.func(cxn, args)


# Set up argument parser
docbits = __doc__.split('\n\n')

argparser = ArgumentParser(description=docbits[0],
                           epilog='\n\n'.join(docbits[-2:]))
argparser.add_argument('-d', '--database',
                       action='store', dest='databasePath',
                       default=os.path.expanduser('~/.oai-harvest/registry.db'),
                       help=("Path to provider registry database. Currently "
                             "supports sqlite3 only.")
                       )
subparsers = argparser.add_subparsers(help='Actions')
# Create the parser for the "add" command
parser_add = subparsers.add_parser('add', help='Add a new OAI-PMH provider')
parser_add.add_argument('name',
                        action='store',
                        help=("Short identifying name for OAI-PMH Provider.")
                        )
parser_add.add_argument('url',
                        action='store',
                        nargs='?',
                        help=("Base URL of OAI-PMH Provider from which to "
                              "harvest.")
                        )
parser_add.add_argument('-p', '--metadataPrefix',
                        action='store', dest='metadataPrefix',
                        default=None,
                        help=("the metadataPrefix of the format (XML Schema) "
                             "in which records should be harvested.")
                        )
group = parser_add.add_mutually_exclusive_group()
group.add_argument('-d', '--dir',
                   action='store', dest='dest',
                   default=None,
                   help=("where to output files for harvested records. "
                         "if not provided, you will be prompted for this "
                         "information")
                   )
parser_add.set_defaults(func=add_provider)
# Create the parser for the "remove" command
parser_rm = subparsers.add_parser('rm',
                                  help='Remove a registered OAI-PMH provider'
                                  )
parser_rm.add_argument('name',
                       action='store',
                       nargs='+',
                       help=("Short identifying name of OAI-PMH Provider "
                             "to remove.")
                       )
parser_rm.set_defaults(func=rm_provider)
# Create the parser for the "list" command
parser_list = subparsers.add_parser('list',
                                    help='List registered OAI-PMH provider'
                                    )
group = parser_list.add_mutually_exclusive_group()
group.add_argument('-u', '--url',
                   action='store_true', dest='url',
                   help="list providers with their base URLs (default)"
                   )
group.add_argument('-d', '--dest',
                   action='store_true', dest='dest',
                   default=False,
                   help="list providers with their destinations"
                   )
group.add_argument('-p', '--metadataPrefix',
                   action='store_true', dest='metadataPrefix',
                   default=False,
                   help="list providers with their metadataPrefixes"
                   )
group.add_argument('-l', '--lastHarvest',
                   action='store_true', dest='lastHarvest',
                   default=False,
                   help=("list providers with the time and date of their "
                         "last completed harvest"
                         )
                   )
parser_list.set_defaults(func=list_providers)

# Check for existence of directory for persistent db, logs etc.
appdir = os.path.expanduser('~/.oai-harvest')
if not os.path.exists(appdir):
    os.mkdir(appdir)

# Set up logger
logging.basicConfig(
    level=logging.DEBUG,
    format='%(asctime)s %(name)-16s %(levelname)-8s %(message)s',
    datefmt='[%Y-%m-%d %H:%M:%S]',
    filename=os.path.join(appdir, 'registry.log')
)
logger = logging.getLogger(__name__)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(levelname)-8s %(message)s')
ch.setFormatter(formatter)
logger.addHandler(ch)


if __name__ == "__main__":
    sys.exit(main())



"""Metadata handling classes."""

import logging

from oaipmh.metadata import MetadataRegistry
from lxml.etree import tostring


class DefaultingMetadataRegistry(MetadataRegistry):
    """MetadataRegistry with default reader and/or writer.

    MetadataRegistry that can be init'd with a default reader and/or writer.
    """

    def __init__(self, defaultReader=None, defaultWriter=None):
        MetadataRegistry.__init__(self)
        self.defaultReader = defaultReader
        self.defaultWriter = defaultWriter

    def readMetadata(self, metadata_prefix, element):
        try:
            return MetadataRegistry.readMetadata(self,
                                                 metadata_prefix,
                                                 element)
        except KeyError as key_error:
            try:
                return self.defaultReader(element)
            except AttributeError:
                raise key_error

    def writeMetadata(self, metadata_prefix, element, metadata):
        try:
            return MetadataRegistry.writeMetadata(self,
                                                  metadata_prefix,
                                                  element,
                                                  metadata
                                                  )
        except KeyError as key_error:
            try:
                return self.defaultWriter(element, metadata)
            except AttributeError:
                raise key_error


class XMLMetadataReader(object):
    """Really simple MetadataReader to serialize metadata to pretty XML."""
    def __call__(self, metadata_element):
        return '\n'.join([tostring(rec_element,
                                   method="xml",
                                   pretty_print=True
                                   )
                          for rec_element
                          in metadata_element])



# encoding: utf-8
"""Harvest records from an OAI-PMH provider.

usage: %prog [-h] [--db DATABASEPATH] [-p METADATAPREFIX]
             [-f YYYY-MM-DD] [-u YYYY-MM-DD] [-s SET] [-d DIR]
             [--delete | --no-delete]
             [-l LIMIT]
             provider [provider ...]

positional arguments:
  provider              OAI-PMH Provider from which to harvest. This may be
                        the base URL of an OAI-PMH server, or the short name
                        of a registered provider. You may also specify "all"
                        for all registered providers.

optional arguments:
  -h, --help            show this help message and exit
  --db DATABASEPATH, --database DATABASEPATH
                        Path to provider registry database. Currently supports
                        sqlite3 only.
  -p METADATAPREFIX, --metadataPrefix METADATAPREFIX
                        the metadataPrefix of the format (XML Schema) in which
                        records should be harvested.
  -f YYYY-MM-DD, --from YYYY-MM-DD
                        harvest only records added/modified after this date.
  -u YYYY-MM-DD, --until YYYY-MM-DD
                        harvest only records added/modified up to this date.
  -s SET, --set SET     harvest only records within this set
  -d DIR, --dir DIR     where to output files for harvested records.default:
                        current working path
  --delete              respect the server's instructions regarding deletions,
                        i.e. delete the files locally (default)
  --no-delete           ignore the server's instructions regarding deletions,
                        i.e. DO NOT delete the files locally
  -l LIMIT, --limit LIMIT
                        place a limit on the number of records to harvest from
                        each provider

Copyright © 2013, the University of Liverpool <http://www.liv.ac.uk>.
All rights reserved.

Distributed under the terms of the BSD 3-clause License
<http://opensource.org/licenses/BSD-3-Clause>.
"""
from __future__ import with_statement, absolute_import

import logging
import os
import platform
import sys
import urllib

from argparse import ArgumentParser
from datetime import datetime

from oaipmh.client import Client
from oaipmh.error import NoRecordsMatchError

from .exceptions import NotOAIPMHBaseURLException
from .metadata import DefaultingMetadataRegistry, XMLMetadataReader
from .registry import verify_database


class OAIHarvester(object):
    """Abstract Base Class for an OAI-PMH Harvester.

    Should be sub-classed in order to do useful things with the harvested
    records (e.g. put them in a directory, VCS repository, local database etc.
    """

    def __init__(self, mdRegistry):
        self._mdRegistry = mdRegistry

    def _listRecords(self, baseUrl, metadataPrefix="oai_dc", **kwargs):
        # Generator to yield records from baseUrl in the given metadataPrefix
        # Add metatdataPrefix to args
        kwargs['metadataPrefix'] = metadataPrefix
        client = Client(baseUrl, metadata_registry)
        # Check that baseUrl actually represents an OAI-PMH target
        try:
            client.identify()
        except IndexError:
            raise NotOAIPMHBaseURLException(
                "{0} does not appear to be an OAI-PMH compatible base URL"
                "".format(baseUrl)
            )
        # Check server timestamp granularity support
        client.updateGranularity()
        for record in client.listRecords(**kwargs):
            yield record

    def harvest(self, baseUrl, metadataPrefix, **kwargs):
        "Harvest records"
        raise NotImplementedError(
            "{0.__class__.__name__} must be sub-classed".format(self)
        )


class DirectoryOAIHarvester(OAIHarvester):
    """OAI-PMH Harvester to output harvested records to files in a directory.

    Directory to output files to is specified at object init/construction
    time.
    """

    def __init__(self, mdRegistry, directory,
                 respectDeletions=True, createSubDirs=False, nRecs=0):
        OAIHarvester.__init__(self, mdRegistry)
        self._dir = os.path.abspath(directory)
        self.respectDeletions = respectDeletions
        self.createSubDirs = createSubDirs
        self.nRecs = nRecs

    def harvest(self, baseUrl, metadataPrefix, **kwargs):
        """Harvest records, return if completed.

        Harvest records, output records to files in the directory and
        return a boolean for whether or not all of the records that the
        server could return were actually stored locally.
        """
        logger = logging.getLogger(__name__).getChild(self.__class__.__name__)
        # A counter for the number of records actually returned
        # enumerate() not used as it would include deleted records
        i = 0
        for header, metadata, about in self._listRecords(
                 baseUrl,
                 metadataPrefix=metadataPrefix,
                 **kwargs):
            if self.nRecs and self.nRecs > 0 and self.nRecs <= i:
                logger.info("Stopping harvest; set limit of {0} has been "
                            "reached".format(self.nRecs))
                break
            filename = "{0}.{1}.xml".format(header.identifier(),
                                            metadataPrefix
                                            )
            protected = []
            if platform.system() != 'Windows':
                protected.append(':')
            if self.createSubDirs:
                if isinstance(self.createSubDirs, basestring):
                    # Replace specified character with platform path separator
                    filename = filename.replace(self.createSubDirs,
                                                os.path.sep
                                                )
                # Do not escape path separators, so that sub-directories
                # can be created
                protected.append(os.path.sep)

            filename = urllib.quote(filename, ''.join(protected))
            fp =  os.path.join(self._dir,
                               filename
                               )
            if not os.path.isdir(os.path.dirname(fp)):
                # Missing base directory or sub-directory
                logger.debug("Creating target directory {0}"
                             "".format(self._dir)
                             )
                os.makedirs(os.path.dirname(fp))

            if not header.isDeleted():
                logger.debug('Writing to file {0}'.format(fp))
                with open(fp, 'w') as fh:
                    fh.write(metadata)
                i += 1
            else:
                if self.respectDeletions:
                    logger.debug("Respecting server request to delete file "
                                 "{0}".format(fp))
                    try:
                        os.remove(fp)
                    except OSError:
                        # File probably does't exist in destination directory
                        # No further action needed
                        pass
                else:
                    logger.debug("Ignoring server request to delete file "
                                 "{0}".format(fp))
        else:
            # Harvesting completed, all available records stored
            return True
        # Loop must have been stopped with ``break``, e.g. due to
        # arbitrary limit
        return False


def main(argv=None):
    """Process command line arguments, harvest records accordingly."""
    global argparser, metadata_registry
    if argv is None:
        args = argparser.parse_args()
    else:
        args = argparser.parse_args(argv)
    logger = logging.getLogger(__name__).getChild('main')
    # Parse from and until into datetime objects
    if args.from_ is not None:
        args.from_ = datetime.strptime(args.from_, "%Y-%m-%d")
    if args.until is not None:
        args.until = datetime.strptime(args.until, "%Y-%m-%d")
    # Establish connection to persistent storage
    cxn = verify_database(args.databasePath)
    # Make a set of providers - don't repeat for repeated arguments
    providers = set(args.provider)
    # Check for "all" providers
    if "all" in args.provider:
        # Remove "all" from set
        providers.remove('all')
        # Update set with all registered providers
        providers.update([row[0]
                          for row
                          in cxn.execute('SELECT name FROM providers')
                          ])
    for provider in providers:
        if not provider.startswith('http://'):
            # Fetch details from provider registry
            cursor = cxn.execute('SELECT url, '
                                 'destination, '
                                 'metadataPrefix, '
                                 'lastHarvest [timestamp]'
                                 'FROM providers '
                                 'WHERE name=?', (provider,))
            row = cursor.fetchone()
            if row is None:
                logger.error("Provider {0} does not exists in database {1}"
                             "".format(provider, args.databasePath))
                continue
            baseUrl = row[0]
            logger.info('Harvesting from registered provider {0} - {1}'
                        ''.format(provider, baseUrl))
            # Allow over-ride of default destination
            if args.dir is not None:
                logger.warning('Value for command line option --dir'
                               ' over-rides registered destination')
            else:
                args.dir = row[1]
            # Allow over-ride of default metadataPrefix
            if args.metadataPrefix is not None:
                logger.warning('Value for command line option --metadataPrefix'
                               ' over-rides registered value')
            else:
                args.metadataPrefix = row[2]
            # Allow over-ride of stored lastHarvest time
            # e.g. to repair some locally munged data
            if args.from_ is not None:
                logger.warning('Value for command line option --from'
                               ' over-rides recorded lastHarvest timestamp')
            else:
                args.from_ = row[3]
        else:
            baseUrl = provider
            logger.info('Harvesting from {0}'.format(baseUrl))
            if args.dir is None:
                args.dir = '.'

        if args.metadataPrefix is None:
            args.metadataPrefix = 'oai_dc'

        # Init harvester object
        harvester = DirectoryOAIHarvester(metadata_registry,
                                          os.path.abspath(args.dir),
                                          respectDeletions=args.deletions,
                                          createSubDirs=args.subdirs,
                                          nRecs=args.limit
                                          )
        # Create a dictionary of keyword args
        # Avoid sending kwargs with value of None - e.g. set=None causes
        # error on servers that don't support set hierarchy.
        kwargs = {}
        if args.from_ is not None:
            kwargs['from_'] = args.from_
        if args.until is not None:
            kwargs['until'] = args.until
            # Set the end time of the harvest slice with which to
            # update the registry if necessary
            lastHarvestEndTime = args.until
        else:
            # Set the end time of the harvest slice to now
            # The first request might create a snapshot of the data on
            # the provider server in order for resumption tokens to work
            # correctly. Any records added after this snapshot, but
            # before completion of harvesting must be included in next
            # harvest.
            lastHarvestEndTime = datetime.now()

        if args.set is not None:
            kwargs['set'] = args.set
        try:
            completed = harvester.harvest(baseUrl,
                                          args.metadataPrefix,
                                          **kwargs
                                          )
        except NoRecordsMatchError:
            # Nothing to harvest
            completed = True
            logger.info("0 records to harvest")
            logger.debug("The combination of the values of the from={0}, "
                         "until={1}, set=(N/A) and metadataPrefix={2} "
                         "arguments results in an empty list."
                         "".format(args.from_,
                                   args.until,
                                   args.metadataPrefix)
                         )
        except Exception as e:
            # Log error
            logger.error(str(e))
            # Continue to next provide without updating database lastHarvest
            continue

        if completed:
            # Update lastHarvest time for registered provider
            with cxn:
                cxn.execute("UPDATE providers SET lastHarvest=? WHERE name=?",
                            (lastHarvestEndTime, provider)
                            )
        else:
            logger.warn("Harvesting incomplete; additional records were "
                        "available from the server")


# Set up argument parser
docbits = __doc__.split('\n\n')

argparser = ArgumentParser(description=docbits[0],
                           epilog='\n\n'.join(docbits[-2:]))
argparser.add_argument('--db', '--database',
                       dest='databasePath',
                       default=os.path.expanduser('~/.oai-harvest/registry.db'),
                       help=("Path to provider registry database. Currently "
                             "supports sqlite3 only.")
                       )
argparser.add_argument('provider',
                       nargs='+',
                       help=("OAI-PMH Provider from which to harvest. This may"
                             " be the base URL of an OAI-PMH server, or the "
                             "short name of a registered provider. You may "
                             "also specify \"all\" for all registered "
                             "providers.")
                       )
argparser.add_argument('-p', '--metadataPrefix',
                       dest='metadataPrefix',
                       help=("the metadataPrefix of the format (XML Schema) "
                             "in which records should be harvested.")
                       )
argparser.add_argument("-f", "--from",
                       dest="from_",
                       metavar="YYYY-MM-DD",
                       help=("harvest only records added/modified after this "
                             "date.")
                       )
argparser.add_argument("-u", "--until",
                       dest="until",
                       metavar="YYYY-MM-DD",
                       help=("harvest only records added/modified up to this "
                             "date.")
                       )
argparser.add_argument("-s", "--set",
                       dest="set",
                       help=("harvest only records within this set")
                       )

group = argparser.add_mutually_exclusive_group()
group.add_argument('-d', '--dir',
                   dest='dir',
                   help=("where to output files for harvested records."
                         "default: current working path")
                   )
# What to do about deletions
group = argparser.add_mutually_exclusive_group()
group.set_defaults(deletions=True)
group.add_argument("--delete",
                   action='store_true',
                   dest='deletions',
                   help=("respect the server's instructions regarding "
                         "deletions, i.e. delete the files locally (default)")
                   )
group.add_argument("--no-delete",
                   action='store_false',
                   dest='deletions',
                   help=("ignore the server's instructions regarding "
                         "deletions, i.e. DO NOT delete the files locally")
                   )
argparser.add_argument("-l", "--limit",
                       dest="limit",
                       type=int,
                       help=("place a limit on the number of records to "
                             "harvest from each provider")
                       )
# What to do about sub-directories
group = argparser.add_mutually_exclusive_group()
group.set_defaults(subdirs=None)
group.add_argument("--create-subdirs",
                   action='store_true',
                   dest='subdirs',
                   help=("create target subdirs (based on / characters in "
                         "identifiers) if they don't exist. To use something "
                         "other than /, use the newer --subdirs-on option"
                         )
                   )
group.add_argument("--subdirs-on",
                   action='store',
                   dest='subdirs',
                   help=("create target subdirs based on occurrences of the "
                         "given character in identifiers"
                         )
                   )


# Set up metadata registry
xmlReader = XMLMetadataReader()
metadata_registry = DefaultingMetadataRegistry(defaultReader=xmlReader)

# Check for existence of directory for persistent db, logs etc.
appdir = os.path.expanduser('~/.oai-harvest')
if not os.path.exists(appdir):
    os.mkdir(appdir)

# Set up logger
logging.basicConfig(
    level=logging.DEBUG,
    format='%(asctime)s %(name)-16s %(levelname)-8s %(message)s',
    datefmt='[%Y-%m-%d %H:%M:%S]',
    filename=os.path.join(appdir, 'harvest.log')
)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(levelname)-8s %(message)s')
ch.setFormatter(formatter)
logging.getLogger(__name__).addHandler(ch)


if __name__ == "__main__":
    sys.exit(main())



"""OAI-PMH Harvesting Exceptions."""


class OAIPMHHarvestException(Exception):
    """Base Class for OAI-PMH Harvesting Exceptions."""
    pass


class NotOAIPMHBaseURLException(OAIPMHHarvestException):
    """URL is not an OAI-PMH base URL."""
    pass



"""Setup for OAI Harvester."""

from __future__ import with_statement

import sys

# Import Distribute / Setuptools
import distribute_setup
distribute_setup.use_setuptools()
from setuptools import setup, find_packages

import oaiharvest

# Check Python version
py_version = getattr(sys, 'version_info', (0, 0, 0))

# Basic information
_name = "oaiharvest"
_version = oaiharvest.__version__
_description = ("A harvester to collect records from an OAI-PMH enabled "
                "provider.")
_author = 'John Harrison'
_author_email = 'john.harrison@liv.ac.uk'

# Find longer description from README
with open('README.rst', 'r') as fh:
    _long_description = fh.read()

# Requirements
with open('requirements.txt', 'r') as fh:
    _install_requires = fh.readlines()

if py_version < (2, 7):
    _install_requires.append('argparse')


# Setup
setup(
    name=_name,
    version=_version,
    description=_description,
    long_description=_long_description,
    packages=[_name],
    author=_author,
    author_email=_author_email,
    maintainer=_author,
    maintainer_email=_author_email,
    include_package_data=True,
    exclude_package_data={'': ['README.*', 'LICENSE.*',
                               'requirements.txt', '.gitignore']},
    requires=['lxml(>=2.1)', 'pyoai(>=2.4)'],
    install_requires=_install_requires,
    setup_requires=['setuptools-git'],
    entry_points={
        'console_scripts': [
            "oai-harvest = oaiharvest.harvest:main",
            "oai-reg = oaiharvest.registry:main"
        ]
    },
    url='http://github.com/bloomonkey/oai-harvest',
    license="BSD",
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7", 
        "Topic :: Text Processing :: Markup",
    ]
)
